==> kjukebox-0.1.2.tar.gz: kjukebox-0.1/lib/yaflib/yaf/frontend/deviceConfig/deviceConfigFifo.cpp <==
/*
  stores deviceConfig data in a fifo. (it makes a deep copy  of the data)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#include <deviceConfig/deviceConfigFifo.h>



DeviceConfigFifo::DeviceConfigFifo() {
  int i;

  freePos=0;
  lWaitForStoppedSource=false;
  lWaitForPausedSource=false;
  removeWhenEmpty=false;
  valid=false;
  lDequeued=false;


  for(i=0;i<_MAX_FIFO_ENTRIES;i++) {
    configQueue[i]=new DeviceConfig();
  }
}


DeviceConfigFifo::~DeviceConfigFifo() {
  int i;

  for(i=0;i<_MAX_FIFO_ENTRIES;i++) {
    delete configQueue[i];
  }
}


  

void DeviceConfigFifo::enqueue(DeviceConfig* config) {
  DeviceConfig* freePosConfig;
  MemChunk* memChunk;
  AudioBuffer* audioBuffer;
  StatusInfo* statusInfo;

  if (freePos == _MAX_FIFO_ENTRIES) {
    cout << "fifo overflow! ignore enqueue"<<endl;
    return;
  }
  freePosConfig=configQueue[freePos];
  config->copyTo(freePosConfig);
  audioBuffer=config->getAudioBuffer();
  memChunk=audioBuffer->getMemChunk();
  memChunk->addLock();
  if (freePos == 0) {
    statusInfo=config->getStatusInfo();
    status=statusInfo->getStatus();
  }

  freePos++;
  if (overflowDanger() == true) {
    DaisyChain* daisyChain=(DaisyChain*)(producer->getDaisyChain());
    daisyChain->addLock(this,producer);
  }
}


DeviceConfig* DeviceConfigFifo::dequeue() {
  MemChunk* memChunk;
  AudioBuffer* audioBuffer;

  audioBuffer=configQueue[0]->getAudioBuffer();
  memChunk=audioBuffer->getMemChunk();

  lDequeued=true;
  return configQueue[0];
}


void DeviceConfigFifo::forwardQueue() {
  int i;
  DeviceConfig* config;
  MemChunk* memChunk;
  AudioBuffer* audioBuffer;
  StatusInfo* statusInfo;

  // first we define the status of the fifo. by default a fifo
  // is active but if we forward the queue and the deviceConfig
  // say that the status is stopped the fifo gets inactive
  // it stays inactive until a deviceConfig is inserted with
  // playing status

  if (freePos == 0) return;
  
  if (lDequeued == false) {
    cout << "error forward without dequeue"<<endl;
  }
  lDequeued=false;
  config=configQueue[0];
  statusInfo=config->getStatusInfo();
  status=statusInfo->getStatus();
  audioBuffer=config->getAudioBuffer();
  memChunk=audioBuffer->getMemChunk();
  memChunk->removeLock();

  for(i=0;i<=freePos-1;i++) {
    configQueue[i]=configQueue[i+1];
  }
  configQueue[freePos]=config;

  freePos--;
  if (overflowDanger() == false) {
    DaisyChain* daisyChain=(DaisyChain*)(producer->getDaisyChain());
    daisyChain->removeLock(this,producer);
  }
}



int DeviceConfigFifo::overflowDanger() {
  return (freePos == (_MAX_FIFO_ENTRIES-2));
}

int DeviceConfigFifo::getFillgrade() {
  return freePos;
}



void DeviceConfigFifo::setValid(int valid) {
  this->valid=valid;
}


int DeviceConfigFifo::getValid() {
  return valid;
}



void DeviceConfigFifo::setStreamProducer(StreamProducer* producer) {
  this->producer=producer;
}

StreamProducer* DeviceConfigFifo::getStreamProducer() {
  return producer;
}


int DeviceConfigFifo::getWaitForStoppedSource() {
  return lWaitForStoppedSource;
}


void DeviceConfigFifo::setWaitForStoppedSource(int lWaitForStopped) {
  this->lWaitForStoppedSource=lWaitForStopped;
}


int DeviceConfigFifo::getWaitForPausedSource() {
  return lWaitForPausedSource;
}


void DeviceConfigFifo::setWaitForPausedSource(int lWaitForPaused) {
  this->lWaitForPausedSource=lWaitForPaused;
}


int DeviceConfigFifo::getDequeue() {
  return lDequeued;
}


int DeviceConfigFifo::getStatus() {
  return status;
}

==> kjukebox-0.3.2.tar.gz: kjukebox-0.3.2/lib/yaflib/yaf/frontend/deviceConfig/deviceConfigFifo.cpp <==
/*
  stores deviceConfig data in a fifo. (it makes a deep copy  of the data)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#include <deviceConfig/deviceConfigFifo.h>
#include "../../../../../config.h"



DeviceConfigFifo::DeviceConfigFifo(NodeDevice* id) {
  int i;

  freePos=0;
  lWaitForStoppedSource=false;
  lWaitForPausedSource=false;
  lDequeued=false;
  this->id=id;
  status=new StatusInfo();

  for(i=0;i<_MAX_FIFO_ENTRIES;i++) {
    configQueue[i]=new DeviceConfig();
  }
}


DeviceConfigFifo::~DeviceConfigFifo() {
  int i;
  
  clear();
  for(i=0;i<_MAX_FIFO_ENTRIES;i++) {
    delete configQueue[i];
  }
  delete status;
  DaisyChain* daisyChain=(DaisyChain*)NodeDevice::getDaisyChain();
  daisyChain->removeAllLocks(this);
}


  

void DeviceConfigFifo::enqueue(DeviceConfig* config) {
  DeviceConfig* freePosConfig;

  if (freePos == _MAX_FIFO_ENTRIES) {
    debugOutput(cout << "fifo overflow! ignore enqueue:"<<getpid()<<endl);
    return;
  }
  StatusInfo* statusInfo=config->getStatusInfo();
  if (statusInfo->getChange()) {
    statusInfo->copyTo(status);
    return;
  }
  freePosConfig=configQueue[freePos];
  config->copyTo(freePosConfig);
  freePosConfig->addLock();
  
  freePos++;
  if (overflowDanger() == true) {
    setOverflowDangerLock();
  }
}


DeviceConfig* DeviceConfigFifo::dequeue() {
  lDequeued=true;
  return configQueue[0];
}


void DeviceConfigFifo::forwardQueue() {
  int i;
  DeviceConfig* config;

  // first we define the status of the fifo. by default a fifo
  // is active but if we forward the queue and the deviceConfig
  // say that the status is stopped the fifo gets inactive
  // it stays inactive until a deviceConfig is inserted with
  // playing status

  if (freePos == 0) {
    debugOutput(cout << "forward queue on empty queue called!"<<endl);
    exit(0);
  }
  if (lDequeued == false) {
    debugOutput(cout << "error forward without dequeue"<<endl);
    exit(0);
  }

  lDequeued=false;
  config=configQueue[0];
  config->removeLock();

  for(i=0;i<=freePos-1;i++) {
    configQueue[i]=configQueue[i+1];
  }
  configQueue[freePos]=config;
  
  freePos--;
  if (overflowDanger() == false) {
    removeOverflowDangerLock();
  } 
}


int DeviceConfigFifo::overflowDanger() {
  return (freePos == (_MAX_FIFO_ENTRIES-2));
}


int DeviceConfigFifo::getFillgrade() {
  return freePos;
}


int DeviceConfigFifo::getWaitForStoppedSource() {
  return lWaitForStoppedSource;
}


void DeviceConfigFifo::setWaitForStoppedSource(int lWaitForStopped) {
  this->lWaitForStoppedSource=lWaitForStopped;
}


int DeviceConfigFifo::getWaitForPausedSource() {
  return lWaitForPausedSource;
}


void DeviceConfigFifo::setWaitForPausedSource(int lWaitForPaused) {
  this->lWaitForPausedSource=lWaitForPaused;
}


int DeviceConfigFifo::getDequeue() {
  return lDequeued;
}


int DeviceConfigFifo::getStatus() {
  return status->getStatus(); 
}


NodeDevice* DeviceConfigFifo::getID() {
  return id;
}


void DeviceConfigFifo::setOverflowDangerLock() {
  if (freePos > 0) {
    DeviceConfig* config=configQueue[0];
    StreamProducer* producer=(StreamProducer*)config->getStreamProducer();
    if (producer == NULL) {
      debugOutput(cout << "fatal Error in DeviceConfigFifo::setOverflowDangerLock!"  << " StreamProducer is NULL!"<<endl );
      exit(-1);
    }
    DaisyChain* daisyChain=(DaisyChain*)(producer->getDaisyChain());
    daisyChain->addLock(this,producer);
  }
}


void DeviceConfigFifo::removeOverflowDangerLock() {
  if (freePos > 0) {
    DeviceConfig* config=configQueue[0];
    StreamProducer* producer=(StreamProducer*)config->getStreamProducer();
     if (producer == NULL) {
      debugOutput(cout << "fatal Error in DeviceConfigFifo::setOverflowDangerLock!"  << " StreamProducer is NULL!"<<endl);
      exit(-1);
    }   
    DaisyChain* daisyChain=(DaisyChain*)(producer->getDaisyChain());
    daisyChain->removeLock(this,producer);
  } 
}

  

void DeviceConfigFifo::clear() {
  while(getFillgrade() > 0) {
    debugOutput(cout << "clear"<<endl);
    dequeue();
    forwardQueue();
  }
  return;
}
